Package: arrayhelpers
Type: Package
Title: Convenience Functions for Arrays
Description: Some convenient functions to work with arrays.
Maintainer: Claudia Beleites <Claudia.Beleites@chemometrix.eu>
Authors@R: c(
    person("Claudia",   "Beleites",  role = c("aut","cre"), email = "Claudia.Beleites@chemometrix.eu")
    )
Version: 1.1-2
Date: 2026-07-24
License: GPL
LazyLoad: yes
Imports: methods, utils, svUnit
URL: http://arrayhelpers.r-forge.r-project.org/
Encoding: UTF-8
Collate: 'unittestdata.R' 'arrayhelpers.R' 'apply.R' 'array2df.R'
        'array2vec.R' 'arrayhelpers-package.R' 'colSums.R'
        'countrows.R' 'delold.R' 'dropdimnames.R' 'ensuredim.R'
        'groupsum.R' 'init.R' 'makeNd.R' 'ndim.R' 'numericindices.R'
        'restoredim.R' 'rowsum.R' 'slice.R' 'stack.R' 'ta.R'
        'unittests.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-24 14:41:13 UTC; cb
Author: Claudia Beleites [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-24 18:50:02 UTC

# Generated by roxygen2: do not edit by hand

export("slice<-")
export(array2df)
export(array2vec)
export(arrayhelpers.unittest)
export(colMeans.AsIs)
export(colSums.AsIs)
export(delold)
export(drop1d)
export(dropdimnames)
export(ensuredim)
export(groupsum)
export(lon)
export(makeNd)
export(ndim)
export(numericindex)
export(restoredim)
export(rowMeans.AsIs)
export(rowSums.AsIs)
export(slice)
export(ta)
export(vec2array)
exportMethods(colMeans)
exportMethods(colSums)
exportMethods(rowMeans)
exportMethods(rowSums)
exportMethods(rowsum)
import(methods)
import(svUnit)
importFrom(utils,head)
importFrom(utils,tail)

### test data
##' @noRd
{
  ## vector
  v <- 1 : 3
  names (v) <- letters [1 : 3]

  ## matrix
  m <- matrix (1:6, 2,
               dimnames = list (rows = letters [1:2],
                 columns = LETTERS [1:3])) 

  ## array
  a <- array (1 : 24, 4 : 2,
              dimnames = list (rows = letters [1:4],
                columns = LETTERS [1:3],
                d3 = 1:2)
              )
}

##' Converting array and vector Indices
##' Calculate the vector index from array indices, and vice versa.
##' 
##' \code{array}s are \code{numeric}s with a \code{dim} attribute and are
##' stored with the first index moving fastest (i.e. by column). They can be
##' indexed both ways.
##' 
##' @aliases array2vec vec2array
##' @param iarr vector with the indices into the array dimensions
##' @param dim vector with the array dimensions, as returned by \code{dim (x)}
##' @return \code{array2vec} returns a scalar, \code{vec2array} a
##'   \code{matrix}.
##' @author C. Beleites
##' @seealso see \code{\link[base]{Extract}} on the difference of indexing an
##'   \code{array} with a vector or a \code{matrix}.
##' @keywords array
##' @export
##' @examples
##' 
##' arr <- array (rnorm (24), dim = 2 : 4)
##' arr
##' 
##' v <- matrix(c(2, 2, 2), nrow = 1)
##' i <- array2vec (v, dim = dim (arr))
##' i
##' arr[v]
##' arr[i]
##' 
##' arr[c(2, 2, 2)] ## indexing with a vector
##' arr[2]
##' 
array2vec <- function (iarr, dim){
  if (!is.matrix (iarr))
    dim (iarr) <- c(1, length (iarr))

  if (ncol (iarr) != length (dim))
    stop ("Number of columns in iarr and number of dimensions differ.")

  if (any (sweep (iarr, 2, dim) > 0))
    stop ("array index > dim")

  pdim <- c(1, cumprod (dim [- length (dim)]))
  iarr <- iarr - 1

  colSums(apply (iarr, 1, "*", pdim)) + 1
}


##' @param ivec scalar with the index into the vector
##' @rdname array2vec
##' @export
##' @examples
##'  
##' i <- 14
##' v <- vec2array (i, dim = dim (arr))
##' v
##' arr [v]
##' arr [i]
##' 
vec2array <- function (ivec, dim) {
  ndim <- length (dim)
  pdim <- c(1, cumprod (dim))

  iarr <- matrix(NA, nrow = length(ivec), ncol = ndim) # matrix for the array indices
  colnames (iarr) <- letters[8 + seq_len (ndim)]       # i, j, k, ...

  ivec <- (ivec - 1)
  for (j in seq_len (ndim))
    iarr [, j] <- (ivec %% pdim [j + 1]) / pdim [j]

  1 + floor(iarr)
}

##' \code{restoredim} restores the shape the array had before \code{makeNd} was called.
##'
##' Attributes \code{old.dim} and \code{old.dimnames} are used by default. \code{restoredim} is the
##' inverse of \code{makeNd}.
##'
##' Note that missing attributes as well as \code{old.dim = NULL} produce a (dimensionless)
##' vector. This is also the case if \code{a} lost the \code{old.*} attributes during 
##' computations like \code{as.numeric}, \code{c}, etc..
##'
##' \code{fromend} together with numeric \code{usedim} specifies dimensions counting from the
##' end. E.g. \code{fromend = TRUE} and \code{usedim = 1 : 3} for an array to be restored to 10d
##' means restoring dimensions 8 : 10. \code{fromend = TRUE} and \code{usedim = -(1 : 3)} restores
##' dimensions 1 to 7.
##' @param old list containing a list with (possibly) elements \code{dim}, \code{dimnames}, and
##' \code{names}. The nth last element of this list is used.
##' @param n how many makeNdim steps to go back?
##' @param ... ignored
##' @param usedim use only the specified dimensions
##' @param fromend if \code{TRUE}, numeric \code{usedim} are counted from the end, see details.
##' @param drop should 1d arrays drop to vectors?
##' @return an array
##' @author Claudia Beleites
##' @rdname makeNd
##' @export
##' @examples
##'
##' a <- array (1 : 24, 4 : 3)
##' a
##' restoredim (makeNd (a, 0))
##'
##' x <- makeNd (a, 0)
##' attr (x, "old")
##' 
restoredim <- function (a, old = NULL, n = 1L, ...,
                        usedim = TRUE, fromend = FALSE, drop = FALSE){

  if (is.null (old)){
    old <- peek (a, "old", n = n)
    a <- pop (a, "old", n = n)
  } else if (!is.null (names (old [[n]])) &&  all (names (old [[n]]) %in% c("dim", "dimnames", "names")) &&
             (is.null (names (old      )) || !all (names (old      ) %in% c("dim", "dimnames", "names")))
             )
      old <- old [[n]]
  ## else assume a list with dim, dimnames and names
  
  if (fromend && is.numeric (usedim))
    usedim <- sort (rev (seq_along (old$dim)) [usedim])
  else
    usedim <- sort (numericindex (x = old$dim, i = usedim, n = names (old$dimnames)))

  a <- structure (a, dim = old$dim [usedim],
                  dimnames =  lon (old$dimnames [usedim]),
                  names =  old$names)

  drop1d (a, drop = drop)
}

.test (restoredim) <- function (){
  checkIdentical (a, restoredim (makeNd (a,  5)))
  checkIdentical (a, restoredim (makeNd (a,  0)))
  
  checkIdentical (a, restoredim (makeNd (a, -5)))
  checkIdentical (a, restoredim (makeNd (a, -2)))

  checkIdentical (v, restoredim (makeNd (v,  0)))
  checkIdentical (v, restoredim (makeNd (v,  1)))
  checkIdentical (v, restoredim (makeNd (v, -1)))
  
  checkIdentical (dim (restoredim (as.numeric (makeNd (a, 0)))), NULL)

  checkIdentical (a, restoredim (restoredim (makeNd (makeNd (a,  5), 0))))
  checkIdentical (a, restoredim (makeNd (makeNd (a,  5), 0), n = 2))

  checkIdentical (a, restoredim (makeNd (makeNd (a,  5), 0), n = 3)) # OK

  warn <- options(warn = 2)$warn
  on.exit (options (warn = warn))
  checkException (restoredim (makeNd (makeNd (a,  5), 0), n = 3))

  tmp <- makeNd (v, 5)
  old <- attr (tmp, "old")
  tmp <- pop (tmp, "old")
  checkIdentical (v, restoredim (tmp, old = old [[1]]))
  checkIdentical (v, restoredim (tmp, old = old))

  ## TODO: test drop and usedim
}

##' Extension of \code{rowsum}
##' 
##' \code{groupsum} extends \code{\link{rowsum}}: it allows \code{group} to be an array of the same shape
##' as \code{x}. 
##' @param x array to be \code{rowsum}med
##' @param group grouping variable (integer or factor) indicating groups of samples. \code{}
##' @param dim along which dimension should the group sums be taken? (default: rows)
##' @param reorder should the groups be ordered? see \code{\link[base]{rowsum}}
##' @param na.rm shoud \code{NA}s be removed?
##' @param ... ignored
##' @param drop should 1d arrays drop to vectors?
##' @return like \code{\link[base]{rowsum}}, but further dimensions of the array are preserved.
##' @author Claudia Beleites
##' @seealso \code{\link[base]{rowsum}} \code{\link{rowsum}}
##' @keywords array algebra arith
##' @export
groupsum <- function(x, group = NULL, dim = 1L, reorder=TRUE, na.rm = FALSE, ...,
                     drop = ! is.array (x)) {
  x <- ensuredim (x)

  ## permute the group dimension to the beginning
  x <- aperm (x, c (dim, seq_len (ndim (x)) [-dim]))

  x <- makeNd (x, 2)
  old <- attr (x, "old") [[1]]
  x <- pop (x, "old")
  

  if (is.null (group)){                 # almost no gain...
    x   <- colSums (x, na.rm = TRUE, drop = FALSE)
  } else if (is.null (dim (group))) {
    if (length (group) == nrow (x)) {
      x   <- rowsum  (x, group = group, na.rm = TRUE)
    } else {
      stop ("wrong length: group ", length (group), " dim: ", nrow (x))
    }   
  } else if (all (dim (group) == dim (x))) {
    stop ("grouping factors of size dim (p) not yet implemented.")
  } 

  old$dim [1] <- nrow (x)
  old$dimnames [1] <- list (rownames (x))

 # mostattributes (x) <- old

  x <- restoredim (x, old = old)
  x <- aperm (x, order (c (dim, seq_len (ndim (x)) [-dim])))

  drop1d (x, drop = drop)
}

.test (groupsum) <- function (){
  groups <- c(2, 1, 2)
  checkEquals (groupsum (a, group = groups, dim = 2),
               structure(c(5 : 8, (5 : 8) * 2L, 17 : 20, (17 : 20) * 2L), 
                           dim = c(4L, 2L, 2L),
                           dimnames = structure(list(rows = letters [1:4],
                             columns = c("1", "2"),
                             d3 = c("1", "2")),
                             names = c("rows", "columns", "d3"))))

  b <- a
  dim (b) <- c (2, 2, 3, 2)
  g <- groupsum (b, group = groups, dim = 3)

  checkEquals (b [,,1,] + b [,,3,], g [,,2,])
  checkEquals (b [,,2,]           , g [,,1,])
  
}

.rowsum <- function(x, group, reorder=TRUE, na.rm = FALSE, ...) {

  x <- makeNd (x, 2)
  old <- attributes (x)

  x <- base::rowsum (x, group = group, reorder = reorder, na.rm = na.rm)

  old$old [[1]]$dim [1] <- nrow (x)
  old$old [[1]]$dimnames [1] <- list (rownames (x))

  mostattributes (x) <- old

  x <- restoredim (x)

  x
}

.test (.rowsum) <- function (){
  rowsum (v, seq_along (v))
}

##' rowsum for arrays
##'
##' This function extends the base function \code{\link[base]{rowsum}}.
##' @param x array to be \code{rowsum}med
##' @param group grouping variable (integer or factor) indicating groups of samples. \code{}
##' @param reorder should the groups be ordered? see \code{\link[base]{rowsum}}
##' @param na.rm shoud \code{NA}s be removed?
##' @param ... ignored
##' @return like \code{\link[base]{rowsum}}, but further dimensions of the array are preserved.
##' @author Claudia Beleites
##' @seealso \code{\link[base]{rowsum}}
##' @keywords array algebra arith
##' @docType methods
##' @rdname rowsum
##' @export
setMethod ("rowsum", signature = signature (x = "array"), .rowsum)

#TODO: test, examples

##' Drop dimnames if all elements are \code{NULL}
##'
##' @param x object
##' @return object without empty dimnames
##' @author Claudia Beleites
##' @export 
dropdimnames <- function (x){
  dimnames (x) <- lon (dimnames (x))

  x
}
##' @rdname dropdimnames
##' @param l list
##' @return \code{lon}: \code{NULL} if all elements of \code{dn} are \code{NULL}, otherwise \code{dn}
##' @export 
lon <- function (l){
  if (all (sapply (l, is.null)))
    NULL
  else
    l
}

##' Enforce array and convert to vector if appropriate
##'
##' \code{ensuredim} turns vectors into 1d-arrays, and leaves arrays unchanged. \code{drop1d} is the
##' inverse: it converts 1d arrays into vectors.
##' @param x vector (or array)
##' @return \code{esuredim} array of at least one dimension
##' @author Claudia Beleites
##' @export 
ensuredim <- function (x){
  if (is.null (dim (x)))
    x <- structure (x, dim = length (x),
                    dimnames =  lon (list (names (x))),
                    names = NULL)
  x
}

.test (ensuredim) <- function (){
  checkEquals (ensuredim (v), structure(1:3, dim = 3L, dimnames = list(c("a", "b", "c"))))
  checkEquals (ensuredim (as.numeric (v)), structure(1:3, dim = 3L))
  checkEquals (ensuredim (m), m)
  checkEquals (ensuredim (a), a)
}

##' @rdname ensuredim
##' @param drop if \code{TRUE}, 1d arrays are converted into vectors
##' @return \code{drop1d} vector, if \code{x} had only 1 dimension
##' @author Claudia Beleites
##' @export 
drop1d <- function (x, drop = TRUE){
  if (drop && ndim (x) == 1)
    x <- structure (x, dim = NULL, dimnames = NULL, names = dimnames (x)[[1]])

  x
}

.test (drop1d) <- function () {
  checkEquals (drop1d (a), a)
  checkEquals (drop1d (m), m)
  checkEquals (drop1d (v), v)
  checkEquals (drop1d (ensuredim (v)), v)
  checkEquals (drop1d (ensuredim (v), drop = FALSE), ensuredim (v))
}

##' @include arrayhelpers.R
##  From base/R/colSums.R
.colSums <- function (x, na.rm = FALSE, dims = 1L, drop = TRUE) {
  if (length (dim (x)) < 2)
    x <- as.matrix (x)

  z <- base::colSums (x = x, na.rm = na.rm, dims = dims)

  if (! drop){
    d  <- dim (x)
    d  [1L : dims] <- 1L

    dn <- dimnames (x)
    dn [1L : dims] <- list (NULL)
    
    z <- structure (z, dim =  d, dimnames = lon (dn))
  }
  
  z
}

.unclasscolSums <- function (x, ...) {
  colSums (unclass (x), ...)
}

.test (.colSums) <- function (){
  ao <- array (1:24, 4:2)
  
  for (d in 1 : 2){
    default <- base::colSums (a, dims = d)
    drop <- colSums (a, dims = d, drop = TRUE)
    nodrop <- colSums (a, dims = d, drop = FALSE)

    checkEquals (default, drop, sprintf ("base version ./. drop = TRUE, dim = %i", d))
    checkEqualsNumeric (c (default), c (nodrop), sprintf ("drop = TRUE ./. FALSE, dim = %i", d))

    dd <- dim (default)
    if (is.null (dd)) dd <- length (default)
    checkEquals (dim (nodrop) [-(1 : d)], dd, sprintf ("result dimensions, d = %i", d))
    checkTrue (all (sapply (dimnames (nodrop) [1 : d], is.null)))
    checkEquals (dimnames (nodrop) [(d + 1) : ndim (nodrop)],
                 dimnames (a)      [(d + 1) : ndim (a)     ])
    nodrop <- colSums (ao, dims = d, drop = FALSE)
    checkEquals (dimnames (nodrop) [(d + 1) : ndim (nodrop)],
                 dimnames (ao)     [(d + 1) : ndim (ao)    ])
  }
}

## TODO: Tests for AsIs, matrix
.colMeans <- function(x, na.rm = FALSE, dims = 1L, drop = TRUE){
  
  if (length (dim (x)) < 2)
    x <- as.matrix (x)

  z <- base::colMeans (x, na.rm = na.rm, dims = dims)

  if (! drop){
    d  <- dim (x)
    d  [1L : dims] <- 1L
    

    dn <- dimnames (x)
    dn [1L : dims] <- list (NULL)
    
    z <- structure (z, dim =  d, dimnames = lon (dn))
  }
  
  z
}

.unclasscolMeans <- function (x, ...) {
  colMeans (unclass (x), ...)
}

.rowSums <- function(x, na.rm = FALSE, dims = 1L, drop = TRUE) {
  if (length (dim (x)) < 2)
    x <- as.matrix (x)

  z <- base::rowSums (x, na.rm = na.rm, dims = dims)

  if (! drop){
    d  <- dim (x)
    d   [(dims + 1L) : length (d)] <- 1L
    
    dn <- dimnames (x)
    dn [(dims + 1L) : length (dn)] <- list (NULL)
    
    z <- structure (z, dim =  d, dimnames = lon (dn))
  }
      
  z
}
.unclassrowSums <- function (x, ...) {
  rowSums (unclass (x), ...)
}

.rowMeans <- function(x, na.rm = FALSE, dims = 1L, drop = TRUE)
{
  if (length (dim (x)) < 2)
    x <- as.matrix (x)

  z <- base::rowMeans (x, na.rm = na.rm, dims = dims)

  if (! drop){
    d  <- dim (x)
    d   [(dims + 1L) : length (d)] <- 1L
    
    dn <- dimnames (x)
    dn [(dims + 1L) : length (dn)] <- list (NULL)
    
    z <- structure (z, dim =  d, dimnames = lon (dn))
  }
      
  z
}
.unclassrowMeans <- function (x, ...) {
  rowMeans (unclass (x), ...)
}

.test (.rowSums) <- function (){
  a <- array (1:24, 4:2)
  for (d in 1 : 2){
    default <- base::rowSums (a, dims = d)
    drop <- rowSums (a, dims = d, drop = TRUE)
    nodrop <- rowSums (a, dims = d, drop = FALSE)

    checkEquals (default, drop, sprintf ("base version ./. drop = TRUE, dim = %i", d))
    checkEquals (c (default), c (nodrop), sprintf ("drop = TRUE ./. FALSE, dim = %i", d))

    dd <- dim (default)
    if (is.null (dd)) dd <- length (default)
    checkEquals (dim (nodrop) [1 : d], dd, sprintf ("result dimensions, d = %i", d))
  }
}

.test (.rowMeans) <- function (){
  a <- array (1:24, 4:2)
  for (d in 1 : 2){
    default <- base::rowMeans (a, dims = d)
    drop <- rowMeans (a, dims = d, drop = TRUE)
    nodrop <- rowMeans (a, dims = d, drop = FALSE)

    checkEquals (default, drop, sprintf ("base version ./. drop = TRUE, dim = %i", d))
    checkEquals (c (default), c (nodrop), sprintf ("drop = TRUE ./. FALSE, dim = %i", d))

    dd <- dim (default)
    if (is.null (dd)) dd <- length (default)
    checkEquals (dim (nodrop) [1 : d], dd, sprintf ("result dimensions, d = %i", d))
  }
}

.test (.colMeans) <- function (){
  a <- array (1:24, 4:2)
  for (d in 1 : 2){
    default <- base::colMeans (a, dims = d)
    drop <- colMeans (a, dims = d, drop = TRUE)
    nodrop <- colMeans (a, dims = d, drop = FALSE)

    checkEquals (default, drop, sprintf ("base version ./. drop = TRUE, dim = %i", d))
    checkEquals (c (default), c (nodrop), sprintf ("drop = TRUE ./. FALSE, dim = %i", d))

    dd <- dim (default)
    if (is.null (dd)) dd <- length (default)
    checkEquals (dim (nodrop) [-(1L : d)], dd, sprintf ("result dimensions, d = %i", d))
  }
}

##' @noRd
setGeneric ("colSums")
##' @noRd
setGeneric ("colMeans")
##' @noRd
setGeneric ("rowSums")
##' @noRd
setGeneric ("rowMeans")

##' Row and column sums and means for numeric arrays. 
##'
##' These functions extend the respective base functions by (optionally) preserving the shape of the
##' array (i.e. the summed dimensions have length 1).
##' 
##' @param x an array of two or more dimensions, containing numeric, complex, integer or logical
##' values, or a numeric data frame.  
##' @param na.rm logical indicating treatment of missing values
##' @param dims integer: Which dimensions are regarded as \sQuote{rows} or \sQuote{columns} to sum
##' over.  For \code{row*}, the sum or mean is  over dimensions \code{dims + 1, \dots}; for \code{col*}
##' it is over  dimensions \code{1 : dims}.
##' @param ... the \code{signature = "AsIs"} methods hand on all parameters
##' @param drop If \code{FALSE}, the number of dimensions is retained: the length of the dimensions
##' that are summed or averaged is set to  1. \code{TRUE} yield the same behaviour as
##' \code{\link[base]{colSums}}
##' @return like \code{\link[base]{colSums}} if \code{drop = TRUE}, otherwise an array where the
##' summed dimensions have length 1.
##' @author Claudia Beleites
##' @seealso \code{\link[base]{colSums}}
##' @keywords array algebra arith
##' @docType methods
##' @rdname colSums
##' @export
##'
##' @examples
##' a <- array (1 : 24, 4 : 2)
##' a
##' 
##' rowSums (a)
##' rowSums (a, drop = FALSE)
##' 
##' colSums (a)
##' colSums (a, drop = FALSE)
##' 
##' colSums (a, dim = 2)
##' colSums (a, dim = 2, drop = FALSE)
##'
setMethod ("colSums", signature = signature (x = "matrix"), .colSums) 
# colSums.matrix <- .colSums              # I still get base::colSums :-(

##' @rdname colSums
##' @export
colSums.AsIs <- .unclasscolSums
# setMethod ("colSums", signature = signature (x = "AsIs"), .unclasscolSums)

##' @rdname colSums
##' @export
setMethod ("colSums", signature = signature (x = "array"), .colSums)

##' @rdname colSums
##' @export
setMethod ("colMeans", signature = signature (x = "matrix"), .colMeans)

##' @rdname colSums
##' @export
colMeans.AsIs <- .unclasscolMeans
##setMethod ("colMeans", signature = signature (x = "AsIs"), .unclasscolMeans)

##' @rdname colSums
##' @export
setMethod ("colMeans", signature = signature (x = "array"), .colMeans)

##' @rdname colSums
##' @export
setMethod ("rowSums", signature = signature (x = "matrix"), .rowSums)

##' @rdname colSums
##' @export
rowSums.AsIs <- .unclassrowSums
#setMethod ("rowSums", signature = signature (x = "AsIs"), .unclassrowSums)

##' @rdname colSums
##' @export
setMethod ("rowSums", signature = signature (x = "array"), .rowSums)

##' @rdname colSums
##' @export
setMethod ("rowMeans", signature = signature (x = "matrix"), .rowMeans)

##' @rdname colSums
##' @export
rowMeans.AsIs <- .unclassrowMeans
##setMethod ("rowMeans", signature = signature (x = "AsIs"), .unclassrowMeans)

##' @rdname colSums
##' @export
setMethod ("rowMeans", signature = signature (x = "array"), .rowMeans)

testAsIs <- function (){
  methods <- c("colSums", "colMeans", "rowSums", "rowMeans")
  for (fn in methods){
    f <- get (fn)
    for (d in 1L : 2L)
      checkEquals (f (a, dims = d), f (I (a), dims = d), msg = sprintf ("AsIs: %s, dims = %i", fn, d))
  }
}

##' Conservative version of apply: tries to preserve the dimensions of the original array
##'
##' \code{\link[base]{apply}} returns its results as an array of shape (not-margin dimensions *
##' length of FUN's result) x MARGIN dimensions.
##'
##' \code{applycons} rearranges these results, so that
##'
##' 1. the result dimensions are appended rather than prepended, and
##' 2. tries to preserve the shape of the 
##' 
##' @param X an array or matrix.
##' @param MARGIN a vector giving the subscripts which the function will be applied over. E.g., for a
##' matrix \code{1} indicates rows, \code{2} indicates columns, \code{c(1, 2)} indicates rows and
##' columns. Where \code{X} has named dimnames, it can be a character vector selecting dimension
##' names.
##' @param FUN the function to be applied: see \sQuote{Details}. In the case of functions like
##' \code{+}, \code{\%*\%}, etc., the  function name must be backquoted or quoted.
##' @param ... optional arguments to \code{FUN}.
##' @return array
##' @author Claudia Beleites
##' @seealso \code{\link[base]{apply}}
##' @include arrayhelpers.R
##' @noRd
.applycons <- function (X, MARGIN, FUN, ...){
    FUN <- match.fun(FUN)

    ## Ensure that X is an array object
    dl <- length(dim(X))
    
    if(!dl) stop("dim(X) must have a positive length")

    if(is.object(X))
      X <- if(dl == 2L) as.matrix(X) else as.array(X)
    ## now record dim as coercion can change it
    ## (e.g. when a data frame contains a matrix).
    d <- dim(X)
    dn <- dimnames(X)
    ds <- seq_len(dl)

    ## Extract the margins and associated dimnames
    if (is.character(MARGIN)) {
        if(is.null(dnn <- names(dn))) # names(NULL) is NULL
           stop("'X' must have named dimnames")
        MARGIN <- match(MARGIN, dnn)
        if (any(is.na(MARGIN)))
            stop("not all elements of 'MARGIN' are names of dimensions")
    }
    
    s.call <- ds [-MARGIN]              # call: with these FUN is called
    s.ans  <- ds [ MARGIN]              # ans:  these stay for the answer
    d.call <- d  [-MARGIN]              # s:    sequence
    d.ans  <- d  [ MARGIN]              # d:    dim
    dn.call<- dn [-MARGIN]              # dn:   dimnames
    dn.ans <- dn [ MARGIN]

    ## do the calls

    d2 <- prod(d.ans)
    if(d2 == 0L) {
        ## arrays with some 0 extents: return ``empty result'' trying
        ## to use proper mode and dimension:
        ## The following is still a bit `hackish': use non-empty X
        newX <- array(vector(typeof(X), 1L), dim = c(prod(d.call), 1L))
        ans <- FUN(if(length(d.call) < 2L) newX[,1] else
                   array(newX[, 1L], d.call, dn.call), ...)
        return(if(is.null(ans)) ans else if(length(d.ans) < 2L) ans[1L][-1L]
               else array(ans, d.ans, dn.ans))
    }
    ## else
    newX <- aperm(X, c(s.call, s.ans))
    dim(newX) <- c(prod(d.call), d2)
    ans <- vector("list", d2)
    if(length(d.call) < 2L) {# vector
        if (length(dn.call)) dimnames(newX) <- c(dn.call, list(NULL))
        for(i in 1L:d2) {
            tmp <- FUN(newX[,i], ...)
            if(!is.null(tmp)) ans[[i]] <- tmp
        }
    } else
       for(i in 1L:d2) {
           tmp <- FUN(array(newX[,i], d.call, dn.call), ...)
           if(!is.null(tmp)) ans[[i]] <- tmp
        }

    ## answer dims and dimnames

    ans.list <- is.recursive(ans[[1L]])
    l.ans <- length(ans[[1L]])

    ans.names <- names(ans[[1L]])
    if(!ans.list)
	ans.list <- any(unlist(lapply(ans, length)) != l.ans)
    if(!ans.list && length(ans.names)) {
        all.same <- vapply(ans, function(x) identical(names(x), ans.names), NA)
        if (!all(all.same)) ans.names <- NULL
    }
    len.a <- if(ans.list) d2 else length(ans <- unlist(ans, recursive = FALSE))
    if(length(MARGIN) == 1L && len.a == d2) {
	names(ans) <- if(length(dn.ans[[1L]])) dn.ans[[1L]] # else NULL
	return(ans)
    }
    if(len.a == d2)
	return(array(ans, d.ans, dn.ans))
    if(len.a && len.a %% d2 == 0L) {
        if(is.null(dn.ans)) dn.ans <- vector(mode="list", length(d.ans))
        dn.ans <- c(list(ans.names), dn.ans)
	return(array(ans, c(len.a %/% d2, d.ans),
		     if(!all(vapply(dn.ans, is.null, NA))) dn.ans))
    }
    return(ans)
}

##' Count equal rows
##'
##' matrices are converted to data.frame.
##' @param x the matrix or data.frame
##' @return data frame with unique rows, their counts and indices into the original data.frame
##' @note this function is subject to changes in the future.
##' @author Claudia Beleites
##' @rdname countrows
##' @importFrom utils head
##' @importFrom utils tail
countRows <- function(x) {
## TODO: derive matrix function
## TODO: generalize to nd arrays
## TODO: tests
## TODO: examples
  
  if (is.matrix (x) && (ncol (x) == 1))
    x <- as.vector (x) 

  order.x <- do.call (order, as.data.frame (x))
  
  if (is.vector (x)) {
    equals.prev <-          x [tail (order.x, -1)  ] == x [head (order.x, -1)  ]
  } else {
    equals.prev <- rowSums (x [tail (order.x, -1), ] != x [head (order.x, -1), ]) == 0
  }

  indices <- split (order.x, cumsum (c (TRUE, ! equals.prev)))

  if (is.vector (x)) {
    x <- x [sapply (indices, function (v) v [[1]]),   drop = FALSE]
  } else {
    x <- x [sapply (indices, function (v) v [[1]]), , drop = FALSE]
  }

  data.frame (cts = sapply (indices, length),
              ind = I (indices),
              x)
}

##' Little helper functions to work with arrays
##' @name arrayhelpers-package
##' @title Package arrayhelpers
##' @docType package
##' @author C. Beleites
##' 
##' Maintainer: Claudia Beleites <claudia.beleites@@chemometrix.eu>
##' @include unittestdata.R
##' @import svUnit
##' @import methods
##' @keywords package
{
  if (!requireNamespace ("svUnit", quietly = TRUE)){
   `.test<-` <- function (f, value) {
      class (value) <-  c ("svTest", "function")
      attr (f, "test") <- value
      f
    }
  } else {
    `.test<-` <- svUnit::`test<-`
  }
}

    

##' A little stack.
##'
##' TODO: implement as reference class?
##' Note: \code{pop} only removes elements. To retrieve them, use \code{peek}.
##' 
##' @param x the object
##' @param an attribute holding the stack
##' @param n numer of element to peek at and numer of elements to pop (delete), respectively
##' @return \code{push} and \code{pop}: the object with stack in list \code{an} pushed/popped by the \code{n} elements
##'
##' \code{peek}: the \code{n}th stack element (without popping!)
##' @author Claudia Beleites
##' @rdname stack
peek <- function (x, an, n = 1L){
  stack <- attr (x, an)
  if (length (stack) == 0) return (NULL)

  if (length (stack) < n){
    n <- length (stack)
    warning ("n set to ", n)
  }

  stack [[n]]
}

##' @rdname stack
pop <- function (x, an, n = 1L){
  stack <- attr (x, an)
  if (length (stack) == 0) return (x)

  if (length (stack) < n){
    n <- length (stack)
    warning ("n set to ", n)
  }

  if (length (stack) == n)
    attr (x, an) <- NULL
  else
    attr (x, an) <- stack [- seq_len (n)]
      
  x
}

##' @usage push (x, an) <- value
##' @rdname stack
##' @param value list of things to push on the stack.
`push<-` <- function (x, an, value){
  stack <- attr (x, an)
  value <- rev (value)                  # expect: last argument first out
  stack <- c (value, stack)             # prepend the new elements

  attr (x, an) <- stack
      
  x
}

##' Run the unit tests
##'
##' Run the unit tests attached to the functions via \link[svUnit]{svUnit} 
##' @return invisibly \code{TRUE} if the tests pass, \code{NA} if \link[svUnit]{svUnit} is not
##' available. Stops if errors are encountered.
##' @author Claudia Beleites
##' @seealso  \link[svUnit]{svUnit} 
##' @export 
##' @import svUnit
arrayhelpers.unittest <- function (){
  if (! requireNamespace ("svUnit", quietly = TRUE)){
    warning ("svUnit required to run the unit tests.")
    return (NA)
  }

  tests <- unlist (eapply (env = getNamespace ("arrayhelpers"), FUN = is.test, all.names = TRUE))
  tests <- names (tests [tests])
  tests <- sapply (tests, get, envir = getNamespace ("arrayhelpers"))

  clearLog ()
  warnlevel <- options()$warn
  options (warn = 0)
  for (t in seq_along (tests))
    runTest (tests [[t]], names (tests) [t])

  options (warn = warnlevel)
  if (interactive ())
    print (stats (Log()))
  else
    print (stats (Log ())[,c ("kind", "msg")])

  errorLog (summarize = FALSE)
  invisible (TRUE)
}

##' Strip the attributes keeping track of the former shape
##'
##' Convenient for printing
##' @param a the array
##' @return \code{a} stripped of the \code{old.*} attributes.
##' @author Claudia Beleites
##' @export
##' @examples
##'
##' a <- arrayhelpers:::a
##' makeNd (a, 2)
##' delold (makeNd (a, 2))
##' 
##' 
delold <- function (a){
   attr (a, "old") <- NULL
   a
}
.test (delold) <- function (){
  tmp <- makeNd (a, 0)
  old <- attributes (tmp)
  old <- old [grepl ("^old", names (old))]
  checkIdentical (old,
                  list (old = list (list (
                          names = NULL, 
                          dimnames = list(rows = c("a", "b", "c", "d"),
                            columns = c("A", "B", "C"), 
                            d3 = c("1", "2")),
                          dim = c(4L, 3L, 2L))))
                  )
  
  tmp <- delold (tmp)
  checkTrue (is.null (attr (old, "old")))
}

.onAttach <- function (libname, pkgname){
  ## unlockBinding(".options", asNamespace("arrayhelpers")) 

  desc <- utils::packageDescription("arrayhelpers")
  vers <- paste("V. ", desc$Version)

  packageStartupMessage ("Package ",  desc$Package, ", version ", desc$Version, "\n\n",
       "If you use this package please cite it appropriately.\n",
       "   citation(\"", desc$Package, "\")\nwill give you the correct reference.", "\n\n",
       "The project homepage is ", desc$URL, "\n\n",
       sep = "")
}

##' Transpose arrays
##'
##' This function provides transposing of arrays or vectors as swapping their first two dimensions.
##' \code{t (array)} can be enabled via \code{\link[methods]{setMethod}}, see the example.
##' @param x an array 
##' @return the array with the first two dimensions swapped.
##' @author Claudia Beleites
##' @seealso \code{\link[base]{t}}
##' @export 
##' @examples
##' a <- array (1 : 24, 4:2)
##' a
##' ta (a)
##' 
##' setMethod ("t", "array", ta)
##' t (a)
##' removeMethod ("t", "array")
##'
ta <- function (x){
  if (! (is.vector (x) || is.matrix (x) || is.array (x)))
      stop ("x must be array, matrix, or vector.")
      
  a <- makeNd (x, 2)                 # ensure at least 2 dimensions

  d <- seq_along (dim (x))
  d [1 : 2] <- 2 : 1                    # swap first 2 dimensins
  
  aperm (x, d)
}

.test (ta) <- function () {
  checkEqualsNumeric (a, ta (ta (a)))
  checkEquals (a, ta (ta (a)))
  checkIdentical (a, ta (ta (a)))


  checkIdentical (ta (m), t (m))

#  instd <- length (findMethod ("t", "array")) > 0

#  if (instd) removeMethod ("t", "array")
#  checkException (t (a))
#  setMethod ("t", "array", ta)
#  checkIdentical (t (t (a)), a)
  
#  if (!instd) removeMethod ("t", "array")
}

##' array2df: Convert multidimensional array into matrix or data.frame
##' The "wide-format" array is converted into a "long-format" \code{matrix} or
##' \code{data.frame}.
##' 
##' If the resulting \code{data.frame} is too large to fit in memory, a
##' \code{matrix} might help.
##' 
##' The main benefit of this function is that it uses matrices as long as
##' possible. This can give large advantages in terms of memory consumption.
##' 
##' @title array2df 
##' @param x \code{array}
##' @param levels \code{list} with the levels for the dimensions of \code{x}.
##' 
##' If \code{levels[[i]]} is \code{NULL} no column is produced for this factor.
##' 
##' If \code{levels[[i]]} is \code{NA}, the result column is a numeric with
##'   range from \code{1} to \code{dim (x)[i]}
##'
##' If  \code{levels[[i]]} is \code{TRUE}, the levels are taken from the dimnames.
##' 
##' \code{names(levels)} yield the resulting column names.
##' @param matrix If \code{TRUE}, a numeric \code{matrix} rather than a
##'   \code{data.frame} is returned.
##' @param label.x Name for the column containing the \code{x} values.
##' @param na.rm should rows where the value of \code{x} is \code{NA} be removed?
##' @return A data.frame or matrix with \code{prod (dim (x))} rows and \code{length (dim (x)) + 1}
##' columns. 
##' @author Claudia Beleites
##' @export
##' @include arrayhelpers.R
##' @keywords array manip
##' @seealso \code{\link[utils]{stack}}
##' @include arrayhelpers.R
##' @examples
##' a <- arrayhelpers:::a
##' a
##' array2df (a)
##' array2df (a, matrix = TRUE)
##' 
##' array2df (a, levels = list(NULL, x = NA, c = NULL), label.x = "value")
##' 
##' array2df (a, levels = list(NULL, x = TRUE, c = c ("foo", "bar")), label.x = "value")
##' 
##' summary (array2df (a,
##'                    levels = list(NULL, x = NA, c = c ("foo", "bar")),
##'                    label.x = "value"))
##' 
##' summary (array2df (a,
##'                    levels = list(NULL, x = NA, c = c ("foo", "bar")),
##'                    label.x = "value",
##'                    matrix = TRUE))
##' 

array2df <- function (x, levels, matrix = FALSE, 
    label.x = deparse (substitute (x)), na.rm = FALSE) {

  dims <- dim (x)
  dn <- dimnames (x)
  if (is.null (dn))
   dn <- lapply (dims, function (x) NULL)

  ## prepare levels if not given
  if (missing (levels)){
    levels <- dn
    levels [sapply (levels, is.null)] <- NA
  }
  
  ## TRUE as abbreviation for "use dimnames"
  i <- sapply (dn, is.null)
  dn [i] <- lapply (dims [i], seq_len)
  i <- sapply (levels, isTRUE)
  levels [i] <- dn [i]
  
  ## get colnames from dimnames of x
  colnames <- names (dn)
  if (! is.null (names (levels))){
    i <- nzchar (names (levels))
    colnames [i] <- names (levels) [i]
  }

  ## prepare unfolding of array
  if (length (levels) != length (dims)) 
    stop ("Levels must have as many elements as x has dimensions.")

  cprod <- c (1, cumprod (dims))
  rprod <- c (rev (cumprod (rev (dims))), 1) [-1]

  ## the dimensions to unfold
  idim <- seq_along (dims) [!sapply (levels, is.null)]

  ## start with a matrix
  df <- matrix (x, nrow = length (x), ncol = length (idim) + 1)

  for (d in seq_along (idim))
    df [, d + 1] <- rep (seq_len (dims [idim [d]]),
                         each = cprod [idim [d]], times = rprod [idim [d]])

  ## delete rows where x is NA?
  if (na.rm) 
    df <- df [! is.na (df [,1]), ]
  
  if (!matrix) {
    df <- as.data.frame (df)

    for (d in seq_along (idim)) {

      ## turn to factor?
      if (!all (is.na (levels [[idim [d]]]))) {
        df [, d + 1] <- factor (df [, d + 1], labels = levels [[idim [d]]])
      }
      
    }
    
  }

  if (!is.null (colnames))
    colnames (df) <- c (label.x, colnames [idim])
  else
    colnames (df) <- c (label.x, paste ("d", idim, sep = ""))
  
  df
}

.test (array2df) <- function (){

  ## no dimnames
  x <- array (1:24, 4:1)
  checkEquals (array2df (x),
               structure (list (x = 1:24,
                                d1 = rep (1:4, 6),
                                d2 = rep (rep (1:3, each = 4), 2),
                                d3 = rep (1:2, each = 12),
                                d4 = rep (1, 24)),
                                names = c("x", "d1", "d2", "d3", "d4"),
                                row.names = c(NA, -24L),
                                class = "data.frame")
               )               

  checkEquals (array2df (x, matrix = TRUE), 
               structure (c (1:24,
                             rep (1:4, 6),
                             rep (rep (1:3, each = 4), 2),
                             rep (1:2, each = 12),
                             rep (1, 24)),
                          dim = c(24L, 5L),
                          dimnames = list(NULL, c("x", "d1", "d2", "d3", "d4")))
               )               
               
  checkEquals (array2df (a), structure(list(a = 1:24, rows = structure(c(1L, 2L, 3L, 4L, 1L, 
    2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 
    2L, 3L, 4L), levels = c("a", "b", "c", "d"), class = "factor"), 
    columns = structure(c(1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 3L, 
    3L, 3L, 3L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 3L, 3L, 3L, 3L
    ), levels = c("A", "B", "C"), class = "factor"), d3 = structure(c(1L, 
    1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 
    2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L), levels = c("1", "2"), class = "factor")), names = c("a", 
    "rows", "columns", "d3"), row.names = c(NA, -24L), class = "data.frame"))

  checkTrue (is.matrix (array2df (a, matrix = TRUE)))

  checkEquals (array2df (a, list (TRUE, NULL, NA)),
               structure(list(a = 1:24, rows = structure(c(1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L,
               3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L), levels = c("a", "b", "c",
               "d"), class = "factor"), d3 = c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 2L,
               2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L)), names = c("a", "rows", "d3"), row.names
               = c(NA, -24L), class = "data.frame")
               )  
}

##' number of dimensions
##'
##' @param a vector, matrix, or array
##' @return integer: length of dim attribute
##' @author Claudia Beleites
##' @export
ndim <- function (a){
  length (dim (a))
}

.test (ndim) <- function (){
  checkEquals (ndim (v), 0L)
  checkEquals (ndim (ensuredim (v)), 1L)
  checkEquals (ndim (m), 2L)
  checkEquals (ndim (a), 3L)
}

##' Convert character or logical indices to numeric
##' @param x the object that is to be indexed
##' @param n names of the object
##' @param i the indices to be converted
##' @return numeric indices
##' @author Claudia Beleites
##' @export 
numericindex <- function (x, i, n = names (x)){
  if (is.character (i))
    match (i, n)
  else if (is.logical (i))
    seq_along (x) [i]
  else if (is.numeric (i))
    i
   else
    stop ("i must be numeric, logical, or character")
}
.test (numericindex) <- function (){
  checkEquals (numericindex (v, c("b", "a", "x")), c (2L, 1L, NA))
  checkEquals (numericindex (v, c(TRUE, FALSE, TRUE)), c(1L, 3L))
  checkEquals (numericindex (v, TRUE), 1 : 3)
  checkEquals (numericindex (v, FALSE), integer (0L))
  checkEquals (numericindex (v, c(TRUE, FALSE)), c(1L, 3L))
  checkEquals (numericindex (v, 1L), 1L)
}

##' Ensure/collapse an array into \code{n} dimensions and restore the old dimensions
##'
##' \code{nameNd} ensures a given number of dimensions:
##' If \code{a} has less than \code{N} dimensions, new dimensions of length 1 are appended.
##' If \code{a} has more than \code{N} dimensions, the supernumerary dimensions are collapsed onto
##' the last dimension.
##'
##' @param a an array (matrix, vector)
##' @param N the desired number of dimensions, 0 to remove the \code{dim} and \code{dimnames}
##' attributes (i.e. to create a vector). 
##' @return N-dimensional array
##' @author Claudia Beleites
##' @export  
##' @examples
##' v <- arrayhelpers:::v
##' v
##' makeNd (v, 1)
##' dim (makeNd (v, 1))
##' dim (makeNd (v, 3))
##' 
##' m <- arrayhelpers:::m
##' m
##' makeNd (m, 1)
##' dim (makeNd (m, 1))
##' makeNd (m, 0) 
##' dim (makeNd (m, 0))
##' makeNd (m, 3)
##' 
##' a <- arrayhelpers:::a
##' a
##' dim (makeNd (a, 1))
##' dim (makeNd (a, 0))
##' makeNd (a,  2)          
##' makeNd (a, -2)
##' makeNd (a, -4)
##' makeNd (a, 3);
##' 
makeNd <- function (a, N) {
   if (! all.equal (N, as.integer (N)))
     warning ("N truncated to integer value")
   N <- as.integer (N)

   d  <- dim (a)
   dn <- dimnames (a)

   ## push the old dimensions to the end of attribute old
   push (a, "old") <- list (list (names = attributes (a)$names, dimnames = dn, dim = d))
                                        # careful: names (a) and attr (a, "names") return dimnames
                                        # [[1]] for 1d arrays! --  We don't want that!
 
   if      (N == 0)        a <- .removedim   (a)
   else if (length (d) <  N && N > 0) a <- .appenddimafter    (a,  N, d, dn)
   else if (length (d) >  N && N > 0) a <- .collapsedimafter  (a,  N, d, dn)
   else if (length (d) < -N && N < 0) a <- .appenddimbefore   (a, -N, d, dn)
   else if (length (d) > -N && N < 0) a <- .collapsedimbefore (a, -N, d, dn)
    
   a
}

.test (makeNd) <- function (){
  checkEqualsNumeric (makeNd (v, 1), v)
  checkEqualsNumeric (makeNd (v, 2), v)
  checkEquals (dim (makeNd (v, 1)), 3L)
  checkEquals (dim (makeNd (v, 2)), c (3L, 1L))
  checkEquals (dimnames (makeNd (v, 1)) [[1]], names (v))

  checkEqualsNumeric (makeNd (m, 3), m)
  checkEquals (dim (makeNd (m, 3)), c(2L, 3L, 1L))
  checkEquals (dimnames (makeNd (m, 3)), c(dimnames (m), list (NULL)))
  
  checkEqualsNumeric (makeNd (a, 0), a)
  checkTrue (is.null (dim (makeNd (a, 0))))
  checkEquals (dim (makeNd (a, 2)), c (4L, 6L))
  checkEquals (dimnames (makeNd (a, 2)), list (rows = letters [1 : 4], NULL))

  checkTrue (is.null (attr (makeNd (ensuredim (v), 2), "old")[[1]]$names),
             msg = "names attribute for 1d arrays")
}

.removedim <- function (a){
  dim (a)      <- NULL
  dimnames (a) <- NULL

  a
}

.appenddimafter <- function (a, N, d, dn){
  if (is.null (d)){   # vector
    d <- length (a)
    dn <- list (names (a))
  }
   
  dim (a)        <- c (d,  rep (1,           N - length (d)))
  if (! is.null (dn))
    dimnames (a) <- c (dn, rep (list (NULL), N - length (dn)))
  
  a
}
.appenddimbefore <- function (a, N, d, dn){
  if (is.null (d)){   # vector
    d <- length (a)
    dn <- list (names (a))
  }
   
  dim (a)        <- c (rep (1,           N - length (d )),  d)
  if (! is.null (dn))
    dimnames (a) <- c (rep (list (NULL), N - length (dn)), dn)
  
  a
}

.collapsedimafter <- function (a, N, d, dn) {
  dim (a)        <- c (d  [seq_len (N - 1)], prod (d [N : length (d)]))
  if (! is.null (dn))
    dimnames (a) <- c (dn [seq_len (N - 1)], list (NULL))

  a  
}

.collapsedimbefore <- function (a, N, d, dn) {
  dim (a)        <- c (prod (d [seq_len (length (d) - N + 1)]), tail (d,  N - 1))
  if (! is.null (dn))
    dimnames (a) <- c (list (NULL),                             tail (dn, N - 1))

  a  
}

##' alternative to index an array
##'
##' \code{slice} is an alternative interface to \code{\link[base]{[}} (extract). 
##' Dimensions to index must be given by name, i for the first, j for the second and so on.
##' @param a vector, matrix, or array
##' @param ... indexing instructions. The names of the arguments specify the dimension 
##'    (i = 1st, j = 2nd, ...). The indexing expressions are the same as for \code{\link[base]{[}}
##' @param value the values to assign 
##' @param drop see \code{\link[base]{[}}
##' @return array
##' @author Claudia Beleites
##' @export 
##' @examples 
##' slice (arrayhelpers:::a, j = 3 : 2)
slice <- function (a, ..., drop = TRUE){
	args <- as.list (rep (TRUE, ndim (a)))
	
	dots <- list (...)
	which <- match (names (dots), letters) - 8
	
	args [which] <- dots
	
	do.call (`[`, c (list (a), args, list (drop = drop)))
}

.test (slice) <- function (){
		checkEquals (slice (a, j = 3 : 2), a [, 3 : 2, ])
		checkEquals (slice (a, j = LETTERS [3 : 2]), a [, 3 : 2, ])
		checkEquals (slice (a, j = LETTERS [3 : 2], i = -2), a [-2, 3 : 2, ])

      checkEquals (slice (a, j = 3, drop = FALSE), a [,3,,drop = FALSE])
}
##' @rdname slice
##' @usage slice (a, ...) <- value
##' @export
##' @examples
##' tmp <- arrayhelpers:::a
##' slice (tmp, j = 2 : 3) <- 0
##' tmp
`slice<-` <- function (a, ..., value){
	args <- as.list (rep (TRUE, ndim (a)))
	
	dots <- list (...)
	which <- match (names (dots), letters) - 8
	
	args [which] <- dots
	
	do.call (`[<-`, c (list (a), args, list (value = value)))
}

.test (`slice<-`) <- function (){

  tmp  <- a
  tmpa <- a
  slice (tmp, j = 2 : 3) <- slice (a, j = 3 : 2)
  tmpa [, 2 : 3 ,] <- a [, 3 : 2,]
  checkEquals (tmp, tmpa, msg = "numeric indexing")

  slice (tmp, j = LETTERS [3 : 2]) <- slice (a, j = 3 : 2)
  tmpa [, LETTERS [3 : 2] ,] <- a [, 3 : 2,]
  checkEquals (tmp, tmpa, msg = "colname indexing")

  
  slice (tmp, j = LETTERS [3 : 2], i = -2) <- slice (a, j = 3 : 2, i = -2)
  tmpa [-2, LETTERS [3 : 2] ,] <- a [-2, 3 : 2,]
  checkEquals (tmp, tmpa, msg = "mixed pos + neg numeric indexing")

  slice (tmp, j = 3) <- 4
  tmpa [,3,]         <- 4
  checkEquals (tmp, tmpa, msg = "single index")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensuredim.R
\name{ensuredim}
\alias{ensuredim}
\alias{drop1d}
\title{Enforce array and convert to vector if appropriate}
\usage{
ensuredim(x)

drop1d(x, drop = TRUE)
}
\arguments{
\item{x}{vector (or array)}

\item{drop}{if \code{TRUE}, 1d arrays are converted into vectors}
}
\value{
\code{esuredim} array of at least one dimension

\code{drop1d} vector, if \code{x} had only 1 dimension
}
\description{
\code{ensuredim} turns vectors into 1d-arrays, and leaves arrays unchanged. \code{drop1d} is the
inverse: it converts 1d arrays into vectors.
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numericindices.R
\name{numericindex}
\alias{numericindex}
\title{Convert character or logical indices to numeric}
\usage{
numericindex(x, i, n = names(x))
}
\arguments{
\item{x}{the object that is to be indexed}

\item{i}{the indices to be converted}

\item{n}{names of the object}
}
\value{
numeric indices
}
\description{
Convert character or logical indices to numeric
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slice.R
\name{slice}
\alias{slice}
\alias{slice<-}
\title{alternative to index an array}
\usage{
slice(a, ..., drop = TRUE)

slice (a, ...) <- value
}
\arguments{
\item{a}{vector, matrix, or array}

\item{...}{indexing instructions. The names of the arguments specify the dimension 
(i = 1st, j = 2nd, ...). The indexing expressions are the same as for \code{\link[base]{[}}}

\item{drop}{see \code{\link[base]{[}}}

\item{value}{the values to assign}
}
\value{
array
}
\description{
\code{slice} is an alternative interface to \code{\link[base]{[}} (extract). 
Dimensions to index must be given by name, i for the first, j for the second and so on.
}
\examples{
slice (arrayhelpers:::a, j = 3 : 2)
tmp <- arrayhelpers:::a
slice (tmp, j = 2 : 3) <- 0
tmp
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ndim.R
\name{ndim}
\alias{ndim}
\title{number of dimensions}
\usage{
ndim(a)
}
\arguments{
\item{a}{vector, matrix, or array}
}
\value{
integer: length of dim attribute
}
\description{
number of dimensions
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/array2df.R
\name{array2df}
\alias{array2df}
\title{array2df}
\usage{
array2df(
  x,
  levels,
  matrix = FALSE,
  label.x = deparse(substitute(x)),
  na.rm = FALSE
)
}
\arguments{
\item{x}{\code{array}}

\item{levels}{\code{list} with the levels for the dimensions of \code{x}.

If \code{levels[[i]]} is \code{NULL} no column is produced for this factor.

If \code{levels[[i]]} is \code{NA}, the result column is a numeric with
  range from \code{1} to \code{dim (x)[i]}

If  \code{levels[[i]]} is \code{TRUE}, the levels are taken from the dimnames.

\code{names(levels)} yield the resulting column names.}

\item{matrix}{If \code{TRUE}, a numeric \code{matrix} rather than a
\code{data.frame} is returned.}

\item{label.x}{Name for the column containing the \code{x} values.}

\item{na.rm}{should rows where the value of \code{x} is \code{NA} be removed?}
}
\value{
A data.frame or matrix with \code{prod (dim (x))} rows and \code{length (dim (x)) + 1}
columns.
}
\description{
array2df: Convert multidimensional array into matrix or data.frame
The "wide-format" array is converted into a "long-format" \code{matrix} or
\code{data.frame}.
}
\details{
If the resulting \code{data.frame} is too large to fit in memory, a
\code{matrix} might help.

The main benefit of this function is that it uses matrices as long as
possible. This can give large advantages in terms of memory consumption.
}
\examples{
a <- arrayhelpers:::a
a
array2df (a)
array2df (a, matrix = TRUE)

array2df (a, levels = list(NULL, x = NA, c = NULL), label.x = "value")

array2df (a, levels = list(NULL, x = TRUE, c = c ("foo", "bar")), label.x = "value")

summary (array2df (a,
                   levels = list(NULL, x = NA, c = c ("foo", "bar")),
                   label.x = "value"))

summary (array2df (a,
                   levels = list(NULL, x = NA, c = c ("foo", "bar")),
                   label.x = "value",
                   matrix = TRUE))

}
\seealso{
\code{\link[utils]{stack}}
}
\author{
Claudia Beleites
}
\keyword{array}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ta.R
\name{ta}
\alias{ta}
\title{Transpose arrays}
\usage{
ta(x)
}
\arguments{
\item{x}{an array}
}
\value{
the array with the first two dimensions swapped.
}
\description{
This function provides transposing of arrays or vectors as swapping their first two dimensions.
\code{t (array)} can be enabled via \code{\link[methods]{setMethod}}, see the example.
}
\examples{
a <- array (1 : 24, 4:2)
a
ta (a)

setMethod ("t", "array", ta)
t (a)
removeMethod ("t", "array")

}
\seealso{
\code{\link[base]{t}}
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countrows.R
\name{countRows}
\alias{countRows}
\title{Count equal rows}
\usage{
countRows(x)
}
\arguments{
\item{x}{the matrix or data.frame}
}
\value{
data frame with unique rows, their counts and indices into the original data.frame
}
\description{
matrices are converted to data.frame.
}
\note{
this function is subject to changes in the future.
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeNd.R, R/restoredim.R
\name{makeNd}
\alias{makeNd}
\alias{restoredim}
\title{Ensure/collapse an array into \code{n} dimensions and restore the old dimensions}
\usage{
makeNd(a, N)

restoredim(
  a,
  old = NULL,
  n = 1L,
  ...,
  usedim = TRUE,
  fromend = FALSE,
  drop = FALSE
)
}
\arguments{
\item{a}{an array (matrix, vector)}

\item{N}{the desired number of dimensions, 0 to remove the \code{dim} and \code{dimnames}
attributes (i.e. to create a vector).}

\item{old}{list containing a list with (possibly) elements \code{dim}, \code{dimnames}, and
\code{names}. The nth last element of this list is used.}

\item{n}{how many makeNdim steps to go back?}

\item{...}{ignored}

\item{usedim}{use only the specified dimensions}

\item{fromend}{if \code{TRUE}, numeric \code{usedim} are counted from the end, see details.}

\item{drop}{should 1d arrays drop to vectors?}
}
\value{
N-dimensional array

an array
}
\description{
\code{nameNd} ensures a given number of dimensions:
If \code{a} has less than \code{N} dimensions, new dimensions of length 1 are appended.
If \code{a} has more than \code{N} dimensions, the supernumerary dimensions are collapsed onto
the last dimension.

Attributes \code{old.dim} and \code{old.dimnames} are used by default. \code{restoredim} is the
inverse of \code{makeNd}.
}
\details{
Note that missing attributes as well as \code{old.dim = NULL} produce a (dimensionless)
vector. This is also the case if \code{a} lost the \code{old.*} attributes during 
computations like \code{as.numeric}, \code{c}, etc..

\code{fromend} together with numeric \code{usedim} specifies dimensions counting from the
end. E.g. \code{fromend = TRUE} and \code{usedim = 1 : 3} for an array to be restored to 10d
means restoring dimensions 8 : 10. \code{fromend = TRUE} and \code{usedim = -(1 : 3)} restores
dimensions 1 to 7.
}
\examples{
v <- arrayhelpers:::v
v
makeNd (v, 1)
dim (makeNd (v, 1))
dim (makeNd (v, 3))

m <- arrayhelpers:::m
m
makeNd (m, 1)
dim (makeNd (m, 1))
makeNd (m, 0) 
dim (makeNd (m, 0))
makeNd (m, 3)

a <- arrayhelpers:::a
a
dim (makeNd (a, 1))
dim (makeNd (a, 0))
makeNd (a,  2)          
makeNd (a, -2)
makeNd (a, -4)
makeNd (a, 3);


a <- array (1 : 24, 4 : 3)
a
restoredim (makeNd (a, 0))

x <- makeNd (a, 0)
attr (x, "old")

}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/array2vec.R
\name{array2vec}
\alias{array2vec}
\alias{vec2array}
\title{Converting array and vector Indices
Calculate the vector index from array indices, and vice versa.}
\usage{
array2vec(iarr, dim)

vec2array(ivec, dim)
}
\arguments{
\item{iarr}{vector with the indices into the array dimensions}

\item{dim}{vector with the array dimensions, as returned by \code{dim (x)}}

\item{ivec}{scalar with the index into the vector}
}
\value{
\code{array2vec} returns a scalar, \code{vec2array} a
  \code{matrix}.
}
\description{
\code{array}s are \code{numeric}s with a \code{dim} attribute and are
stored with the first index moving fastest (i.e. by column). They can be
indexed both ways.
}
\examples{

arr <- array (rnorm (24), dim = 2 : 4)
arr

v <- matrix(c(2, 2, 2), nrow = 1)
i <- array2vec (v, dim = dim (arr))
i
arr[v]
arr[i]

arr[c(2, 2, 2)] ## indexing with a vector
arr[2]

 
i <- 14
v <- vec2array (i, dim = dim (arr))
v
arr [v]
arr [i]

}
\seealso{
see \code{\link[base]{Extract}} on the difference of indexing an
  \code{array} with a vector or a \code{matrix}.
}
\author{
C. Beleites
}
\keyword{array}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unittests.R
\name{arrayhelpers.unittest}
\alias{arrayhelpers.unittest}
\title{Run the unit tests}
\usage{
arrayhelpers.unittest()
}
\value{
invisibly \code{TRUE} if the tests pass, \code{NA} if \link[svUnit]{svUnit} is not
available. Stops if errors are encountered.
}
\description{
Run the unit tests attached to the functions via \link[svUnit]{svUnit}
}
\seealso{
\link[svUnit]{svUnit}
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowsum.R
\docType{methods}
\name{rowsum,array-method}
\alias{rowsum,array-method}
\title{rowsum for arrays}
\usage{
\S4method{rowsum}{array}(x, group, reorder = TRUE, na.rm = FALSE, ...)
}
\arguments{
\item{x}{array to be \code{rowsum}med}

\item{group}{grouping variable (integer or factor) indicating groups of samples. \code{}}

\item{reorder}{should the groups be ordered? see \code{\link[base]{rowsum}}}

\item{na.rm}{shoud \code{NA}s be removed?}

\item{...}{ignored}
}
\value{
like \code{\link[base]{rowsum}}, but further dimensions of the array are preserved.
}
\description{
This function extends the base function \code{\link[base]{rowsum}}.
}
\seealso{
\code{\link[base]{rowsum}}
}
\author{
Claudia Beleites
}
\keyword{algebra}
\keyword{arith}
\keyword{array}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/delold.R
\name{delold}
\alias{delold}
\title{Strip the attributes keeping track of the former shape}
\usage{
delold(a)
}
\arguments{
\item{a}{the array}
}
\value{
\code{a} stripped of the \code{old.*} attributes.
}
\description{
Convenient for printing
}
\examples{

a <- arrayhelpers:::a
makeNd (a, 2)
delold (makeNd (a, 2))


}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dropdimnames.R
\name{dropdimnames}
\alias{dropdimnames}
\alias{lon}
\title{Drop dimnames if all elements are \code{NULL}}
\usage{
dropdimnames(x)

lon(l)
}
\arguments{
\item{x}{object}

\item{l}{list}
}
\value{
object without empty dimnames

\code{lon}: \code{NULL} if all elements of \code{dn} are \code{NULL}, otherwise \code{dn}
}
\description{
Drop dimnames if all elements are \code{NULL}
}
\author{
Claudia Beleites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayhelpers.R, R/arrayhelpers-package.R
\docType{package}
\name{arrayhelpers-package}
\alias{arrayhelpers-package}
\title{Package arrayhelpers}
\description{
Little helper functions to work with arrays

Some convenient functions to work with arrays.
}
\seealso{
Useful links:
\itemize{
  \item \url{http://arrayhelpers.r-forge.r-project.org/}
}

}
\author{
C. Beleites

Maintainer: Claudia Beleites <claudia.beleites@chemometrix.eu>

\strong{Maintainer}: Claudia Beleites \email{Claudia.Beleites@chemometrix.eu}

Authors:
\itemize{
  \item Claudia Beleites \email{Claudia.Beleites@chemometrix.eu}
}

}
\keyword{internal}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colSums.R
\docType{methods}
\name{colSums,matrix-method}
\alias{colSums,matrix-method}
\alias{colSums.AsIs}
\alias{colSums,array-method}
\alias{colMeans,matrix-method}
\alias{colMeans.AsIs}
\alias{colMeans,array-method}
\alias{rowSums,matrix-method}
\alias{rowSums.AsIs}
\alias{rowSums,array-method}
\alias{rowMeans,matrix-method}
\alias{rowMeans.AsIs}
\alias{rowMeans,array-method}
\title{Row and column sums and means for numeric arrays.}
\usage{
\S4method{colSums}{matrix}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

colSums.AsIs(x, ...)

\S4method{colSums}{array}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

\S4method{colMeans}{matrix}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

colMeans.AsIs(x, ...)

\S4method{colMeans}{array}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

\S4method{rowSums}{matrix}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

rowSums.AsIs(x, ...)

\S4method{rowSums}{array}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

\S4method{rowMeans}{matrix}(x, na.rm = FALSE, dims = 1L, drop = TRUE)

rowMeans.AsIs(x, ...)

\S4method{rowMeans}{array}(x, na.rm = FALSE, dims = 1L, drop = TRUE)
}
\arguments{
\item{x}{an array of two or more dimensions, containing numeric, complex, integer or logical
values, or a numeric data frame.}

\item{na.rm}{logical indicating treatment of missing values}

\item{dims}{integer: Which dimensions are regarded as \sQuote{rows} or \sQuote{columns} to sum
over.  For \code{row*}, the sum or mean is  over dimensions \code{dims + 1, \dots}; for \code{col*}
it is over  dimensions \code{1 : dims}.}

\item{drop}{If \code{FALSE}, the number of dimensions is retained: the length of the dimensions
that are summed or averaged is set to  1. \code{TRUE} yield the same behaviour as
\code{\link[base]{colSums}}}

\item{...}{the \code{signature = "AsIs"} methods hand on all parameters}
}
\value{
like \code{\link[base]{colSums}} if \code{drop = TRUE}, otherwise an array where the
summed dimensions have length 1.
}
\description{
These functions extend the respective base functions by (optionally) preserving the shape of the
array (i.e. the summed dimensions have length 1).
}
\examples{
a <- array (1 : 24, 4 : 2)
a

rowSums (a)
rowSums (a, drop = FALSE)

colSums (a)
colSums (a, drop = FALSE)

colSums (a, dim = 2)
colSums (a, dim = 2, drop = FALSE)

}
\seealso{
\code{\link[base]{colSums}}
}
\author{
Claudia Beleites
}
\keyword{algebra}
\keyword{arith}
\keyword{array}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupsum.R
\name{groupsum}
\alias{groupsum}
\title{Extension of \code{rowsum}}
\usage{
groupsum(
  x,
  group = NULL,
  dim = 1L,
  reorder = TRUE,
  na.rm = FALSE,
  ...,
  drop = !is.array(x)
)
}
\arguments{
\item{x}{array to be \code{rowsum}med}

\item{group}{grouping variable (integer or factor) indicating groups of samples. \code{}}

\item{dim}{along which dimension should the group sums be taken? (default: rows)}

\item{reorder}{should the groups be ordered? see \code{\link[base]{rowsum}}}

\item{na.rm}{shoud \code{NA}s be removed?}

\item{...}{ignored}

\item{drop}{should 1d arrays drop to vectors?}
}
\value{
like \code{\link[base]{rowsum}}, but further dimensions of the array are preserved.
}
\description{
\code{groupsum} extends \code{\link{rowsum}}: it allows \code{group} to be an array of the same shape
as \code{x}.
}
\seealso{
\code{\link[base]{rowsum}} \code{\link{rowsum}}
}
\author{
Claudia Beleites
}
\keyword{algebra}
\keyword{arith}
\keyword{array}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stack.R
\name{peek}
\alias{peek}
\alias{pop}
\alias{push<-}
\title{A little stack.}
\usage{
peek(x, an, n = 1L)

pop(x, an, n = 1L)

push (x, an) <- value
}
\arguments{
\item{x}{the object}

\item{an}{attribute holding the stack}

\item{n}{numer of element to peek at and numer of elements to pop (delete), respectively}

\item{value}{list of things to push on the stack.}
}
\value{
\code{push} and \code{pop}: the object with stack in list \code{an} pushed/popped by the \code{n} elements

\code{peek}: the \code{n}th stack element (without popping!)
}
\description{
TODO: implement as reference class?
Note: \code{pop} only removes elements. To retrieve them, use \code{peek}.
}
\author{
Claudia Beleites
}
